"use strict";
var core_1 = require('@angular/core');
var http_1 = require('@angular/http');
var marked = require('marked');
require('rxjs/add/observable/throw');
require('rxjs/add/operator/catch');
require('rxjs/add/operator/map');
var Observable_1 = require('rxjs/Observable');
var marked_options_1 = require('./marked-options');
var MarkdownService = (function () {
    function MarkdownService(http, options) {
        this.http = http;
        this.options = options;
        if (!this.renderer) {
            this.renderer = new marked.Renderer();
        }
    }
    Object.defineProperty(MarkdownService.prototype, "renderer", {
        get: function () {
            return this.options.renderer;
        },
        set: function (value) {
            this.options.renderer = value;
        },
        enumerable: true,
        configurable: true
    });
    MarkdownService.prototype.compile = function (markdown, markedOptions) {
        if (markedOptions === void 0) { markedOptions = this.options; }
        var precompiled = this.precompile(markdown);
        return marked(precompiled, markedOptions);
    };
    MarkdownService.prototype.getSource = function (src) {
        var _this = this;
        return this.http
            .get(src)
            .catch(function (error) { return _this.handleError(error); })
            .map(function (data) { return _this.extractData(data); })
            .map(function (markdown) { return _this.handleExtension(src, markdown); });
    };
    MarkdownService.prototype.highlight = function () {
        if (!window['Prism']) {
            return;
        }
        window['Prism'].highlightAll(false);
    };
    MarkdownService.prototype.extractData = function (response) {
        return response.text() || '';
    };
    MarkdownService.prototype.handleError = function (error) {
        var errMsg;
        if (error instanceof http_1.Response) {
            var body = error.json() || '';
            var err = body.error || JSON.stringify(body);
            errMsg = error.status + " - " + (error.statusText || '') + " " + err;
        }
        else {
            errMsg = error.message ? error.message : error.toString();
        }
        console.error(errMsg);
        return Observable_1.Observable.throw(errMsg);
    };
    MarkdownService.prototype.handleExtension = function (src, markdown) {
        var extension = src
            ? src.split('.').splice(-1).join()
            : null;
        return extension !== 'md'
            ? '```' + extension + '\n' + markdown + '\n```'
            : markdown;
    };
    MarkdownService.prototype.precompile = function (markdown) {
        if (!markdown) {
            return '';
        }
        var indentStart;
        return markdown
            .replace(/\&gt;/g, '>')
            .split('\n')
            .map(function (line) {
            // find position of 1st non-whitespace character
            // to determine the markdown indentation start
            if (line.length > 0 && isNaN(indentStart)) {
                indentStart = line.search(/\S|$/);
            }
            // remove whitespaces before indentation start
            return indentStart
                ? line.substring(indentStart)
                : line;
        }).join('\n');
    };
    MarkdownService.decorators = [
        { type: core_1.Injectable },
    ];
    /** @nocollapse */
    MarkdownService.ctorParameters = function () { return [
        { type: http_1.Http, },
        { type: marked_options_1.MarkedOptions, },
    ]; };
    return MarkdownService;
}());
exports.MarkdownService = MarkdownService;
//# sourceMappingURL=markdown.service.js.map